package com.upay.sdk.balance;

import com.alibaba.fastjson.JSONObject;
import com.upay.sdk.exception.HmacVerifyException;
import com.upay.sdk.exception.RequestException;
import com.upay.sdk.exception.ResponseException;
import com.upay.sdk.exception.UnknownException;
import com.upay.sdk.executer.ResultListenerAdpater;
import com.upay.sdk.serviceprovider.v3_0.declaration.builder.SubMerchantBalanceBuilder;
import com.upay.sdk.serviceprovider.v3_0.declaration.executer.SubMerchantBalanceExecuter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SubMerchantBalanceQueryServlet extends HttpServlet {


    static final Logger LOGGER = LoggerFactory.getLogger(SubMerchantBalanceQueryServlet.class);
    private static final long serialVersionUID = -679848935977904231L;

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        doPost(req, resp);
    }

    @Override
    protected void doPost(final HttpServletRequest req, final HttpServletResponse resp)
            throws ServletException, IOException {

        final String agent = req.getHeader("User-agent");//判断客户端
        LOGGER.info("agent:{}", agent);

        //商户通过此接口查询所有子商户的汇总余额，merchantId上传服务商商编号
        String merchantId = req.getParameter("merchantId");
        final PrintWriter out = resp.getWriter();
        if (StringUtils.isBlank(merchantId)) {
            out.println("参数异常</br>");
            out.println("商户编号不能为空");
            return;
        } else {
            if (merchantId.length() != 9) {
                out.println("参数异常</br>");
                out.println("商户编号长度必须为9");
                return;
            }
        }

        //订单号
        String requestId = req.getParameter("requestId");
        //币种
        String currency = req.getParameter("currency");
        //账户类型
        String accountType = req.getParameter("accountType");
        //时间戳
        //String dateTime = req.getParameter("dateTime");
        //为方便测试，时间戳自动取系统时间，-2秒，避免本地时间快于服务器时间
        String dateTime = DateFormatUtils.format(DateUtils.addSeconds(new Date(), -2), "yyyy-MM-dd HH:mm:ss");
        SubMerchantBalanceBuilder builder = new SubMerchantBalanceBuilder(merchantId);

        builder.setRequestId(requestId);
        builder.setCurrency(currency);
        builder.setAccountType(accountType);
        builder.setDateTime(dateTime);

        try {
            SubMerchantBalanceExecuter executer = new SubMerchantBalanceExecuter();
            executer.query(builder, new ResultListenerAdpater() {
                @Override
                public void success(JSONObject jsonObject) {
                    out.println("处理成功！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void failure(JSONObject jsonObject) {
                    out.println("处理失败！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }

                @Override
                public void pending(JSONObject jsonObject) {
                    out.println("待处理！<br/>");
                    String result = jsonObject.toJSONString();
                    out.print(result);
                }
            });
        } catch (ResponseException e) {
            out.println("响应异常</br>");
            out.println(e.toString());
        } catch (HmacVerifyException e) {
            out.println("签名验证异常</br>");
            out.println(e.toString());
        } catch (RequestException e) {
            out.println("请求异常</br>");
            out.println(e.toString());
        } catch (UnknownException e) {
            out.println("未知异常</br>");
            out.println(e.toString());
        }
    }
}
